package eu.dnetlib.scholix.api.index;

import org.apache.commons.pool2.PooledObject;
import org.apache.commons.pool2.PooledObjectFactory;
import org.apache.commons.pool2.impl.DefaultPooledObject;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.client.RestHighLevelClient;
import org.springframework.data.elasticsearch.client.ClientConfiguration;
import org.springframework.data.elasticsearch.client.RestClients;

public class ElasticSearchClientFactory implements PooledObjectFactory<RestHighLevelClient> {

    private ElasticSearchProperties elasticSearchProperties;


    public ElasticSearchClientFactory(final  ElasticSearchProperties elasticSearchProperties){
        this.elasticSearchProperties = elasticSearchProperties;

    }

    public PooledObject<RestHighLevelClient> makeObject() throws Exception {



        final ClientConfiguration clientConfiguration = ClientConfiguration.builder()
                .connectedTo(elasticSearchProperties.getClusterNodes().split(","))
                .withConnectTimeout(elasticSearchProperties.getConnectionTimeout())
                .withSocketTimeout(elasticSearchProperties.getSocketTimeout())
                .build();
        RestHighLevelClient cc = RestClients.create(clientConfiguration).rest();


        return new DefaultPooledObject(cc);
    }

    public void destroyObject(PooledObject<RestHighLevelClient> pooledObject) throws Exception {
        RestHighLevelClient client = pooledObject.getObject();
        if(client!=null&&client.ping(RequestOptions.DEFAULT)){
            try {
                client.close();
            }catch (Exception e){
                //ignore
            }
        }
    }

    public boolean validateObject(PooledObject<RestHighLevelClient> pooledObject) {
        RestHighLevelClient client = pooledObject.getObject();
        try {
            return client.ping(RequestOptions.DEFAULT);
        }catch(Exception e){
            return false;
        }
    }

    public void activateObject(PooledObject<RestHighLevelClient> pooledObject) throws Exception {
        RestHighLevelClient client = pooledObject.getObject();
        boolean response = client.ping(RequestOptions.DEFAULT);
    }

    public void passivateObject(PooledObject<RestHighLevelClient> pooledObject) throws Exception {
        //nothing
    }


}